@@ -4,7 +4,7 @@ package App::ElasticSearch::Utilities::QueryString;
 use strict;
 use warnings;
 
-our $VERSION = '8.2'; # VERSION
+our $VERSION = '8.3'; # VERSION
 
 use App::ElasticSearch::Utilities qw(:config);
 use App::ElasticSearch::Utilities::Query;
@@ -160,7 +160,7 @@ App::ElasticSearch::Utilities::QueryString - CLI query string fixer
 
 =head1 VERSION
 
-version 8.2
+version 8.3
 
 =head1 SYNOPSIS
 
@@ -402,6 +402,58 @@ Which would expand to:
 This option will iterate through the whole file and unique the elements of the list.  They will then be transformed into
 an appropriate L<terms query|http://www.elasticsearch.org/guide/en/elasticsearch/reference/current/query-dsl-terms-query.html>.
 
+=head3 Wildcards
+
+We can also have a group of wildcard or regexp in a file:
+
+    $ cat wildcards.dat
+    *@gmail.com
+    *@yahoo.com
+
+To enable wildcard parsing, prefix the filename with a C<*>.
+
+    es-search.pl to_address:*wildcards.dat
+
+Which expands the query to:
+
+    {
+      "bool": {
+        "minimum_should_match":1,
+        "should": [
+           {"wildcard":{"to_outbound":{"value":"*@gmail.com"}}},
+           {"wildcard":{"to_outbound":{"value":"*@yahoo.com"}}}
+        ]
+      }
+    }
+
+No attempt is made to verify or validate the wildcard patterns.
+
+=head3 Regular Expressions
+
+If you'd like to specify a file full of regexp, you can do that as well:
+
+    $ cat regexp.dat
+    .*google\.com$
+    .*yahoo\.com$
+
+To enable regexp parsing, prefix the filename with a C<~>.
+
+    es-search.pl to_address:~regexp.dat
+
+Which expands the query to:
+
+    {
+      "bool": {
+        "minimum_should_match":1,
+        "should": [
+          {"regexp":{"to_outbound":{"value":".*google\\.com$"}}},
+          {"regexp":{"to_outbound":{"value":".*yahoo\\.com$"}}}
+        ]
+      }
+    }
+
+No attempt is made to verify or validate the regexp expressions.
+
 =head2 App::ElasticSearch::Utilities::QueryString::Nested
 
 Implement the proposed nested query syntax early.  Example:
@@ -414,7 +466,7 @@ Brad Lhotsky <brad@divisionbyzero.net>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is Copyright (c) 2021 by Brad Lhotsky.
+This software is Copyright (c) 2022 by Brad Lhotsky.
 
 This is free software, licensed under:
 
