@@ -4,7 +4,7 @@ package App::ElasticSearch::Utilities::QueryString::FileExpansion;
 use strict;
 use warnings;
 
-our $VERSION = '8.2'; # VERSION
+our $VERSION = '8.3'; # VERSION
 
 use CLI::Helpers qw(:output);
 use File::Slurp::Tiny qw(read_lines);
@@ -29,9 +29,34 @@ my %parsers = (
 sub handle_token {
     my($self,$token) = @_;
 
+    my $makeMatcher = sub {
+        my ($matcher,$field,$patterns)  = @_;
+        my @tests;
+        foreach my $pattern (@{ $patterns }) {
+            push @tests, { $matcher => { $field => { value => $pattern } } };
+        }
+        return {
+            bool => {
+                should => \@tests,
+                minimum_should_match => 1,
+            }
+        }
+    };
+    my %make = (
+        terms => sub {
+            my ($field, $uniq) = @_;
+            return { terms => { $field => $uniq } };
+        },
+        regexp   => sub { $makeMatcher->(regexp   => @_) },
+        wildcard => sub { $makeMatcher->(wildcard => @_) },
+    );
     if( my ($term,$match) = split /\:/, $token, 2 ) {
         if( defined $match && $match =~ /(.*\.(\w{3,4}))(?:\[([^\]]+)\])?$/) {
             my($file,$type,$col) = ($1,$2,$3);
+            # Support Wildcards
+            my $matcher = $file =~ s/^\~// ? 'regexp'
+                        : $file =~ s/^\*// ? 'wildcard'
+                        : 'terms';
             $col //= -1;
             $type = lc $type;
             verbose({level=>2,color=>'magenta'}, sprintf "# %s attempt of %s type, %s[%s] %s",
@@ -46,7 +71,8 @@ sub handle_token {
                         $col,
                         scalar(keys %$uniq),
                     );
-                    return [{condition => {terms => {$term => [sort keys %$uniq]}}}];
+                    my $qs = [ sort keys %{ $uniq } ];
+                    return [{condition => $make{$matcher}->($term,$qs) }];
                 }
             }
         }
@@ -146,7 +172,7 @@ App::ElasticSearch::Utilities::QueryString::FileExpansion - Build a terms query
 
 =head1 VERSION
 
-version 8.2
+version 8.3
 
 =head1 SYNOPSIS
 
@@ -222,6 +248,58 @@ Which would expand to:
 This option will iterate through the whole file and unique the elements of the list.  They will then be transformed into
 an appropriate L<terms query|http://www.elasticsearch.org/guide/en/elasticsearch/reference/current/query-dsl-terms-query.html>.
 
+=head3 Wildcards
+
+We can also have a group of wildcard or regexp in a file:
+
+    $ cat wildcards.dat
+    *@gmail.com
+    *@yahoo.com
+
+To enable wildcard parsing, prefix the filename with a C<*>.
+
+    es-search.pl to_address:*wildcards.dat
+
+Which expands the query to:
+
+    {
+      "bool": {
+        "minimum_should_match":1,
+        "should": [
+           {"wildcard":{"to_outbound":{"value":"*@gmail.com"}}},
+           {"wildcard":{"to_outbound":{"value":"*@yahoo.com"}}}
+        ]
+      }
+    }
+
+No attempt is made to verify or validate the wildcard patterns.
+
+=head3 Regular Expressions
+
+If you'd like to specify a file full of regexp, you can do that as well:
+
+    $ cat regexp.dat
+    .*google\.com$
+    .*yahoo\.com$
+
+To enable regexp parsing, prefix the filename with a C<~>.
+
+    es-search.pl to_address:~regexp.dat
+
+Which expands the query to:
+
+    {
+      "bool": {
+        "minimum_should_match":1,
+        "should": [
+          {"regexp":{"to_outbound":{"value":".*google\\.com$"}}},
+          {"regexp":{"to_outbound":{"value":".*yahoo\\.com$"}}}
+        ]
+      }
+    }
+
+No attempt is made to verify or validate the regexp expressions.
+
 =for Pod::Coverage handle_token
 
 =head1 AUTHOR
@@ -230,7 +308,7 @@ Brad Lhotsky <brad@divisionbyzero.net>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is Copyright (c) 2021 by Brad Lhotsky.
+This software is Copyright (c) 2022 by Brad Lhotsky.
 
 This is free software, licensed under:
 
